@@ -2,19 +2,24 @@
 
 import re
 import sys
+import asyncio
+import requests
+from threading import Thread
 from pathlib import Path
 from pyuca import Collator
-
-from PyQt5.QtCore import QTranslator, QUrl, Qt, QAbstractTableModel, QSize
-from PyQt5.QtWidgets import QApplication, QMainWindow, QFileDialog, QHeaderView, QDesktopWidget
-from PyQt5.QtGui import QDesktopServices, QIcon, QColor
+from pyperclip import copy as pyclipcopy
+from qrcode import QRCode, ERROR_CORRECT_L
+from PyQt5.QtCore import QTranslator, QUrl, Qt, QAbstractTableModel
+from PyQt5.QtWidgets import QApplication, QMainWindow, QFileDialog, QHeaderView, QDialog
+from PyQt5.QtGui import QDesktopServices, QIcon, QColor, QPixmap
 
 sys.path.append(Path(__file__).parent.parent.as_posix()) # https://stackoverflow.com/questions/16981921
 from negar.virastar import PersianEditor, UnTouchable
-from negar.constants import INFO
+from negar.constants import INFO, __version__ as negar__version
 from negar_gui.constants import __version__, LOGO
 from negar_gui.Ui_mwin import Ui_MainWindow
 from negar_gui.Ui_uwin import Ui_uwWindow
+from negar_gui.Ui_hwin import Ui_Dialog
 
 # ########################################################################
 # # IMPORT Custom widgets
@@ -37,7 +42,11 @@ def init_decorator(func):
         if self.width() > w or self.height() > h:
             self.showMaximized()
         # set layout direction automatically
-        if parent: self.centralwidget.setLayoutDirection(parent.layoutDirection())
+        if parent:
+            try:
+                self.centralwidget.setLayoutDirection(parent.layoutDirection())
+            except: # QDialog  has no centralwidget
+                pass
         # connect widgets to their proper slot
         self.connectSlots()
     return wrapper
@@ -135,14 +144,35 @@ class UntouchWindow(QMainWindow, Ui_uwWindow):
             self.parent.show()
             self.parent.edit_text()
 
+class HelpWindow(QDialog, Ui_Dialog):
+    @init_decorator
+    def __init__(self, parent=None, title=None, label=None):
+        self.parent = parent
+        super(HelpWindow, self).__init__(parent)
+        self.setupUi(self)
+        self.setWindowTitle(title)
+        if isinstance(label, str):
+            self.label.setText(label)
+        if isinstance(label, QPixmap):
+            self.label.setPixmap(label)
+            # self.setFixedSize(QSize(600,600))
+
+
+    def connectSlots(self):
+        pass
+
+    def keyPressEvent(self, event):
+        if event.key() == Qt.Key.Key_Escape:
+            self.close()
+        else:
+            super().keyPressEvent(event)
+
 
 class MyWindow(QMainWindow, Ui_MainWindow):
     @init_decorator
     def __init__(self, parent=None):
         super(MyWindow, self).__init__(parent)
         self.setupUi(self)
-        self.vertical_btn.setEnabled(False)
-        self.horizontal_btn.setEnabled(False)
         try:
             # with open(f"{NEGARGUIPATH}/style.qss") as style:
             #     self.setStyleSheet(style.read())
@@ -156,16 +186,38 @@ class MyWindow(QMainWindow, Ui_MainWindow):
         # destination language
         self.dest = "en"
         # ui language : 0->en, 1->fa
-        self.lan = 'English'
+        self.lang = 'English'
         self.editing_options = []
         self.clipboard = QApplication.clipboard()
         self.fileDialog = QFileDialog()
+        self._statusBar()
+        # Checks for new release
+        Thread(target=lambda: asyncio.run(self.updateCheck()), daemon=True).start()
 
+    async def updateCheck(self):
+        nurl  = 'https://raw.github.com/shahinism/python-negar/master/negar/constants.py'
+        ngurl = 'https://raw.github.com/javadr/negar-gui/master/negar_gui/constants.py'
+        async def get(link):
+            with requests.get(link) as response:
+                return re.search(r'(__version__ = "(\d\.\d(\.\d+)?)")', response.text, re.M).group(2)
+        negar_t = asyncio.create_task(get(nurl))
+        negargui_t = asyncio.create_task(get(ngurl))
+        negar_v = await negar_t
+        negargui_v = await negargui_t
+        version = lambda v: list(map(int,v.split('.')))
+        negar_nv, negargui_nv = [version(i) for i in (negar_v, negargui_v)]
+        negar_ov, negargui_ov = [version(i) for i in (negar__version, __version__)]
+        notification = ''
+        message = 'New version is available for {}. Use `pip install --upgrade {}` to update'
+        if negar_nv>negar_ov:
+            notification = message.format('negar', 'python-negar')
+        if negargui_nv>negargui_ov:
+            notification = message.format('negar-gui', 'negar-gui')
+        self._statusBar(f"{notification}" if notification!=message else '')
 
     def connectSlots(self):
         self.autoedit_handler()
         # connect to slots
-        self.clipboard.dataChanged.connect(self.onClipboradChanged)
         self.autoedit_chkbox.stateChanged.connect(self.autoedit_handler)
         self.edit_btn.clicked.connect(self.edit_text)
         self.actionOpen.triggered.connect(self.openFileSlot)
@@ -181,10 +233,12 @@ class MyWindow(QMainWindow, Ui_MainWindow):
         self.actionPersian.triggered.connect(lambda: self.changeLanguage('Persian'))
         self.actionEnglish.triggered.connect(lambda: self.changeLanguage('English'))
 
-        self.actionNegar_Help.setShortcut("F1")
         self.actionNegar_Help.triggered.connect(lambda: self.input_editor.setText(INFO) )
-        self.actionAbout.setShortcut("CTRL+H")
-        self.actionAbout.triggered.connect(
+        self.actionAbout_Negar.setShortcut("CTRL+H")
+        self.actionAbout_Negar.triggered.connect(
+            lambda: (HelpWindow(parent=self, title='About Negar', label=self.edit_text(INFO)).show())
+        )
+        self.actionAbout_Negar_GUI.triggered.connect(
             lambda: QDesktopServices.openUrl(QUrl("https://github.com/javadr/negar-gui"))
         )
         self.actionDonate.triggered.connect(
@@ -214,6 +268,80 @@ class MyWindow(QMainWindow, Ui_MainWindow):
         self.actionExaggerating_ZWNJ.triggered.connect(lambda: (self.option_control(), self.autoedit_handler()))
 
         self.actionUntouchable_Words.triggered.connect(lambda: (UntouchWindow(parent=self).show(), MainWindow.hide()))
+        self.actionCopy.triggered.connect(self.copySlot)
+        self.copy_btn.clicked.connect(self.copySlot)
+
+        self.actionInteractive_Clipboard.triggered.connect(lambda:
+            self.clipboard.dataChanged.connect(self.onClipboardChanged) if self.actionInteractive_Clipboard.isChecked()
+            else self.clipboard.dataChanged.disconnect(self.onClipboardChanged))
+        self.actionQr_Code.triggered.connect(self.qrcode)
+
+        # Change GridLayout Orientation
+        def grid_layout(layout='h'):
+            if layout=='v':
+                self.gridLayout.setHorizontalSpacing(5)
+            elif layout == 'h':
+                self.gridLayout.setVerticalSpacing(0)
+            widgets = (self.input_editor_label, self.input_editor,
+                        self.output_editor_label, self.output_editor)
+            for widget in widgets:
+                self.gridLayout.removeWidget(widget)
+                widget.setParent(None)
+            elements = (
+                (0, 0, 1, 1),  # Position: 0x0 1 rowspan 1 colspan
+                (1, 0, 1, 1),  # Position: 1x0 1 rowspan 1 colspan
+                (0, 1, 1, 1),  # Position: 0x1 1 rowspan 1 colspan
+                (1, 1, 1, 1),  # Position: 1x1 1 rowspan 1 colspan
+            ) if layout=='v' else (
+                (0, 0, 1, 2),  # Position: 0x0 1 rowspan 2 colspan
+                (1, 0, 1, 2),  # Position: 1x0 1 rowspan 2 colspan
+                (2, 0, 1, 2),  # Position: 2x0 1 rowspan 2 colspan
+                (3, 0, 1, 2),  # Position: 3x0 1 rowspan 2 colspan
+            )
+            for i, widget in enumerate(widgets):
+                self.gridLayout.addWidget(widget, *elements[i])
+
+        self.vertical_btn.clicked.connect(lambda: (self.actionSide_by_Side_View.setChecked(True), grid_layout('v')) )
+        self.horizontal_btn.clicked.connect(lambda: (self.actionSide_by_Side_View.setChecked(False), grid_layout('h')) )
+        self.actionSide_by_Side_View.triggered.connect(lambda:
+            grid_layout('v') if self.actionSide_by_Side_View.isChecked() else grid_layout('h'))
+
+        def grid_full_input():
+            widgets = (self.output_editor_label, self.output_editor)
+            for widget in widgets:
+                self.gridLayout.removeWidget(widget)
+                widget.setParent(None)
+        self.actionFull_Screen_Input.triggered.connect(lambda: (
+            grid_full_input() if self.actionFull_Screen_Input.isChecked() else
+                grid_layout('v') if self.actionSide_by_Side_View.isChecked() else grid_layout('h') )
+        )
+
+    def qrcode(self):
+        if len(self.output_editor.toPlainText().strip())==0:
+            if self.lang == 'Persian':
+                self._statusBar('هیچ متنی برای نمایش از طریق کد QR وجود ندارد!')
+            else: # English
+                self._statusBar('There is no text to to feed the QR Code!')
+            return
+        qr = QRCode(
+            version = 1,
+            error_correction = ERROR_CORRECT_L,
+            box_size = 10,
+            border = 4
+        )
+        qr.add_data(self.output_editor.toPlainText())
+        qr.make(fit = True)
+        img = qr.make_image()
+        img.save('/tmp/test.png')
+        pixmap = QPixmap('/tmp/test.png')
+        screen = QApplication.desktop().screenGeometry()
+        w = min(screen.height(), screen.width())
+        if w-90 < img.size[0]:
+            pixmap = pixmap.scaled(w-90, w-90, Qt.KeepAspectRatio)
+        HelpWindow(parent=self, title='Qr Code', label=pixmap).show()
+
+    def _statusBar(self, notification=''):
+        self.statusBar.showMessage(f'Negar v{negar__version} [[Negar-GUI v{__version__}]] {notification}')
 
     def openFileSlot(self):
         filename, filetype = self.fileDialog.getOpenFileName(self, "Open File - A Plain Text", ".")
@@ -236,20 +364,23 @@ class MyWindow(QMainWindow, Ui_MainWindow):
                 except Exception as e:
                     self.output_editor.setPlainText(e.args[1])
 
-    def changeLanguage(self, lan):
+    def changeLanguage(self, lang):
         """
         change ui language
         """
-        if lan=='Persian' and self.lan!='Persian':
-            self.lan = 'Persian'
+        if lang=='Persian' and self.lang!='Persian':
+            self.lang = 'Persian'
             self.trans.load("fa", directory=f"{NEGARGUIPATH}/ts")
             self.setLayoutDirection(Qt.LayoutDirection.RightToLeft)
             self.centralwidget.setLayoutDirection(Qt.LayoutDirection.RightToLeft)
-        elif lan=='English' and self.lan!='English':
-            self.lan = 'English'
+            self.autoedit_chkbox.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
+            self.statusBar.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
+        elif lang=='English' and self.lang!='English':
+            self.lang = 'English'
             self.trans.load("en")
             self.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
             self.centralwidget.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
+            self.autoedit_chkbox.setLayoutDirection(Qt.LayoutDirection.RightToLeft)
         else:
             return
         _app = QApplication.instance()
@@ -267,18 +398,21 @@ class MyWindow(QMainWindow, Ui_MainWindow):
         s = self.output_editor.toPlainText()
         if s:
             QApplication.clipboard().setText(s)
+        return s
 
-    def onClipboradChanged(self):
+    def onClipboardChanged(self):
         text = self.clipboard.text()
         if text:
             self.input_editor.setPlainText(text)
             self._set_font_size()
 
+    def closeEvent(self, event):
+        # event = QEvent(QEvent.Clipboard)
+        # QApplication.sendEvent(QApplication.clipboard(), event)
+        pyclipcopy(self.copySlot())
+
     def keyPressEvent(self, event):
         if event.key() == Qt.Key.Key_Escape:
-            sanitizedText = self.output_editor.toPlainText()
-            if sanitizedText:
-                self.destroyed.connect(lambda: QApplication.clipboard().setText(sanitizedText) )
             self.close()
         elif event.key() == Qt.Key.Key_F11:
             if self.isMaximized():
@@ -357,14 +491,17 @@ class MyWindow(QMainWindow, Ui_MainWindow):
         if not self.actionExaggerating_ZWNJ.isChecked():
             self.editing_options.append("exaggerating-zwnj")
 
-    def edit_text(self):
-        self.output_editor.clear()
-        run_PE = PersianEditor(self.input_editor.toPlainText(), *self.editing_options)
-        self.output_editor.append(run_PE.cleanup())
+    def edit_text(self, text=None):
+        if not text:
+            self.output_editor.clear()
+            run_PE = PersianEditor(self.input_editor.toPlainText(), *self.editing_options)
+            self.output_editor.append(run_PE.cleanup())
+        else:
+            return PersianEditor(text, *self.editing_options).cleanup()
 
 def main():
-    app = QApplication(sys.argv)
     global MainWindow
+    app = QApplication(sys.argv)
     MainWindow = MyWindow()
     MainWindow.show()
     sys.exit(app.exec_())
